gen_pis= function(p_ORR,p_TOX,phi){
  ## Generate cell probabilities for multinomial dist
  stopifnot(phi>0)
  pi=matrix(NA,ncol=1,nrow=4)
  ## Quadratic equation coefs
  quad_a = 1-phi
  quad_b = 1-(1-phi)*(p_ORR+p_TOX)
  quad_c= -phi*p_ORR*p_TOX
  if(phi!=1){
    pi[1,1] = (-quad_b+sqrt(quad_b**2-4*quad_a*quad_c))/2/quad_a# quadratic formula
  }
  else{
    pi[1,1] = p_ORR*p_TOX
  }
  pi[2,1] = p_ORR - pi[1,1]
  pi[3,1] = p_TOX - pi[1,1]
  pi[4,1] = phi*pi[2,1]*pi[3,1]/pi[1,1]
  return(pi)
}

### Trial simulation
Trial_sim = function(pi_with_phi,n.sim,L,n.interim){
  raw_sim_array = array(rmultinom(n=n.sim*L,size = n.interim,p=pi_with_phi),dim = c(4,L,n.sim))
  sim_array = apply(apply(raw_sim_array,c(1,3),cumsum),c(1,3),t)
  return(sim_array)
}

### BOPII early termination foreach
Trial_cal =function(data_temp,lambda,nu,collapse_ORR,collapse_TOX,
                    n0=1,pi_with_phi,p_ORR_0,p_TOX_0){
  Reject_H0=0
  L = ncol(data_temp)
  trans_mat= rbind(c(1,0,0,0),c(1,1,0,0),c(1,0,1,0),c(-2,0,0,1))
  a = abs(n0*trans_mat%*%pi_with_phi)
  ## interim analysis
  for(L_i in 1:L){
    X_temp = data_temp[,L_i]
    cutoff_temp_ORR = 1-lambda*(L_i/L)**(nu)
    cutoff_temp_TOX = 1-lambda*(L_i/L)**(nu/3)
    pos_prob_ORR = pbeta(p_ORR_0,shape1 = t(a+X_temp)%*%collapse_ORR,
                           shape2 =t(a+X_temp)%*%(1-collapse_ORR))
    pos_prob_TOX = 1-pbeta(p_TOX_0,shape1 = t(a+X_temp)%*%collapse_TOX,
                           shape2 =t(a+X_temp)%*%(1-collapse_TOX))
    if((pos_prob_ORR>cutoff_temp_ORR)|(pos_prob_TOX>cutoff_temp_TOX)){
      Reject_H0=0
      break
    }
    else{
      if(L_i==L){
        Reject_H0=1
      }
      else{
        next
      }
    }
  }
  
  # outcome = c(early_stop,Reject_H0)
  return(Reject_H0)
}

### function for vectorization foreach
Trial_sim_foreach = function(lambda,nu,sim_array_test,
                            Trial_cal = Trial_cal,
                            collapse_ORR,collapse_TOX,
                            n0=1,pi_with_phi,p_ORR_0,p_TOX_0){
  outcome_mat=apply(sim_array_test,3,Trial_cal,lambda =lambda, nu= nu,collapse_ORR = collapse_ORR,
                    collapse_TOX = collapse_TOX,n0=1,pi_with_phi=pi_with_phi,
                    p_ORR_0=p_ORR_0,
                    p_TOX_0=p_TOX_0)
  return(mean(outcome_mat))
}

### Stopping boundary calculation
stopBoundary_cal =function(lmd_nu_opt,L,n,p_ORR_0,p_TOX_0,
                           collapse_ORR,collapse_TOX,pi_with_phi,n0=1){
  trans_mat= rbind(c(1,0,0,0),c(1,1,0,0),c(1,0,1,0),c(-2,0,0,1))
  a = abs(n0*trans_mat%*%pi_with_phi)
  stopBoundary_ORR = c()
  stopBoundary_TOX =c()
  for(L_i in 1:L){
    cutoff_temp_ORR=1-lmd_nu_opt[1]*(L_i/L)**lmd_nu_opt[2]
    cutoff_temp_TOX=1-lmd_nu_opt[1]*(L_i/L)**(lmd_nu_opt[2]/3)
    pos_prob_temp_ORR=pbeta(p_ORR_0,shape1 = sum(a%*%collapse_ORR)+0:(n*L_i),shape2 = sum(a%*%(1-collapse_ORR))+(n*L_i):0)
    if(sum(pos_prob_temp_ORR>cutoff_temp_ORR)!=0){
      b.max_ORR=max(which(pos_prob_temp_ORR>cutoff_temp_ORR))+1;
      stopBoundary_ORR= c(stopBoundary_ORR,(0:(n*L_i))[b.max_ORR])
    }
    else
      stopBoundary_ORR= c(stopBoundary_ORR,0)
    
    
    pos_prob_temp_TOX=1-pbeta(p_TOX_0,shape1 = sum(a%*%collapse_TOX)+0:(n*L_i),shape2 = sum(a%*%(1-collapse_TOX))+(n*L_i):0)
    if(sum(pos_prob_temp_TOX)!=0){
      b.min_TOX=min(which(pos_prob_temp_TOX>cutoff_temp_TOX))+1;
      stopBoundary_TOX= c(stopBoundary_TOX,(0:(n*L_i))[b.min_TOX])
    }
    else{
      stopBoundary_TOX = c(stopBoundary_TOX,n*L_i)
    }
  }
  stopBoundary_mat = cbind((1:L)*n,stopBoundary_ORR,stopBoundary_TOX)
  colnames(stopBoundary_mat) = c("acc_num_patients"," <=ORR bound"," >=TOX bound")
  return(stopBoundary_mat)
}

## Power cal with stopping boundary
power_cal = function(data_temp,stopBoundary_mat){
  stopBoundary_ORR = stopBoundary_mat[,2]
  stopBoundary_TOX = stopBoundary_mat[,3]
  Reject_H0=0
  L = nrow(stopBoundary_mat)
  ## interim analysis
  for(L_i in 1:L){
    X_temp = data_temp[,L_i]
    
    if((X_temp%*%collapse_ORR<=stopBoundary_ORR[L_i])|(X_temp%*%collapse_TOX>=stopBoundary_TOX[L_i])){
      Reject_H0=0
      break
    }
    else{
      if(L_i==L){
        Reject_H0=1
      }
      else{
        next
      }
    }
  }
  return(Reject_H0)
}


### Comparison stage

## posterior probability of ORR_H - ORR_L or TOX_L-TOX_H using a double integral
compare_pos_cal = function(eta,x_1,x_2,collapse_type,pi_with_phi_1,pi_with_phi_2,n0=1,interval_div = 20){
  trans_mat= rbind(c(1,0,0,0),c(1,1,0,0),c(1,0,1,0),c(-2,0,0,1))
  a_1 = abs(n0*trans_mat%*%pi_with_phi_1)
  a_2 = abs(n0*trans_mat%*%pi_with_phi_2)
  intervals = seq(0,1,length.out = interval_div+1)
  integral_sum = 0
  for(interval_div_i in 1:interval_div){
    pos_prob_temp = integrate(function(x) (1-pbeta(eta+x,
                                                   shape1 =t(a_1+x_1)%*%collapse_type,
                                                   shape2 = t(a_1+x_1)%*%(1-collapse_type)))*
                                dbeta(x,shape1=t(a_2+x_2)%*%collapse_type,
                                      shape2 = t(a_2+x_2)%*%(1-collapse_type)),intervals[interval_div_i],intervals[interval_div_i+1])
    integral_sum = integral_sum + pos_prob_temp$value
  }

                              
  ### Prob(p_1-p_2>=eta|x_1,x_2) = integrate P(p_1>p_2+eta|p_2,x_1)p(p_2|x_2)from 0 to 1
  return(integral_sum) 
}

## Stopboundary for comparison
stopBoundary_cal_compare=function(theta2_ORR,theta2_TOX,case_id,sim_array_High_Low_Compare_list,
                                  pi_with_phi_High_list,
                                  pi_with_phi_Low_list,stopBoundary_mat_opt,n,WithDetails=FALSE){

  
  sim_array_High_Low_Compare_case = sim_array_High_Low_Compare_list[[case_id]]
  pi_with_phi_Low_case = pi_with_phi_Low_list[[case_id]]
  pi_with_phi_High_case = pi_with_phi_High_list[[case_id]]
  
  
  stats_vec=c()
  ## Need to soft code case_i
  for(i in 1:n.sim){
    data_temp = sim_array_High_Low_Compare_case[,,i,]
    
    Termin_H = FALSE
    Termin_L = FALSE
    Termin_H_only = FALSE
    Termin_L_only = FALSE
    Termin_H_reason = NA
    Termin_L_reason = NA
    Termin_BOTH =FALSE
    Termin_None = FALSE
    n.sample.size_High_dose = 0
    n.sample.size_Low_dose = 0
    n.sample.size = 0
    for(L_i in 1:L){
      if(Termin_H==FALSE|Termin_L==FALSE){
        if(Termin_H ==FALSE){
          #Single BOPII
          High_temp = data_temp[,L_i,1]
          n.sample.size_High_dose =n.sample.size_High_dose+n
          if(High_temp%*%collapse_ORR<=stopBoundary_mat_opt[L_i,2] | 
             High_temp%*%collapse_TOX>=stopBoundary_mat_opt[L_i,3] ){
            Termin_H = TRUE
            Termin_H_reason= paste0("BOPII_termin_interim_",L_i)
          }
        }
        if(Termin_L ==FALSE){
          #Single BOPII
          Low_temp = data_temp[,L_i,2]
          n.sample.size_Low_dose =n.sample.size_Low_dose+n
          if(Low_temp%*%collapse_ORR<=stopBoundary_mat_opt[L_i,2] | 
             Low_temp%*%collapse_TOX>=stopBoundary_mat_opt[L_i,3] ){
            Termin_L = TRUE
            Termin_L_reason= paste0("BOPII_termin_interim_",L_i)
          }
        } ## Complete patients collection at L_i-th interim and single-arm BOPII
        
        ## Comparison if both pass the futility test
        if(Termin_H==FALSE & Termin_L ==FALSE){
          ## Need to soft code pi_with_phi_1 and **_2.
          pos_prob_compare_ORR_temp = compare_pos_cal(eta = eta_ORR,x_1 = High_temp,x_2 =Low_temp,
                                                      collapse_type = collapse_ORR,
                                                      pi_with_phi_1 =  pi_with_phi_High_case,
                                                      pi_with_phi_2 =  pi_with_phi_Low_case,n0 = 1
          ) 
          # pos_prob_compare_ORR_temp is P(ORR_h - ORR_l > eta_ORR | X)
          
          pos_prob_compare_TOX_temp = 1-compare_pos_cal(eta = eta_TOX,x_1 = High_temp,x_2 =Low_temp,
                                                        collapse_type = collapse_TOX,
                                                        pi_with_phi_1 =  pi_with_phi_High_case,
                                                        pi_with_phi_2 =  pi_with_phi_Low_case,n0 = 1
          )
          # pos_prob_compare_TOX_temp is P(TOX_h - TOX_l < eta_TOX | X)
          
          if(pos_prob_compare_TOX_temp>theta2_TOX[L_i] & (pos_prob_compare_ORR_temp>theta2_ORR[L_i])){
            ## Low dose is terminated
            Termin_L = TRUE
            Termin_L_reason= paste0("DOD_termin_interim_",L_i)
          }
          if((1-pos_prob_compare_TOX_temp>theta2_TOX[L_i]) & (1-pos_prob_compare_ORR_temp>theta2_ORR[L_i])){
            ## High dose is terminated
            Termin_H = TRUE
            Termin_H_reason= paste0("DOD_termin_interim_",L_i)
          }
          
        }
        
        if(L_i==L){
          Termin_None = (1-Termin_L)*(1-Termin_H)
          Termin_L_only =(Termin_L)*(1-Termin_H)
          Termin_H_only=(Termin_H)*(1-Termin_L)
          Termin_BOTH = (Termin_L)*(Termin_H)
          n.sample.size = n.sample.size + n.sample.size_High_dose+n.sample.size_Low_dose
        }
        
      }
      else{ ## When two doses are terminated already
        Termin_None = (1-Termin_L)*(1-Termin_H)
        Termin_L_only =(Termin_L)*(1-Termin_H)
        Termin_H_only=(Termin_H)*(1-Termin_L)
        Termin_BOTH = (Termin_L)*(Termin_H)
        n.sample.size = n.sample.size + n.sample.size_High_dose+n.sample.size_Low_dose
        break
      }
    }
    stats =c(Termin_L_only,Termin_H_only,Termin_None,Termin_BOTH,n.sample.size,n.sample.size_High_dose,n.sample.size_Low_dose,
             Termin_H_reason,Termin_L_reason,theta2_ORR,theta2_TOX,i,case_id)
    stats_vec = rbind(stats_vec,stats)
    
  }
  
  
  colnames(stats_vec) = c("Recommend High only","Recommend Low only","Recommend Both","Recommend None",
                               "n_sample_size","n_sample_size_High_dose","n_sample_size_Low_dose","Termin_H_reason",
                               "Termin_L_reason",paste0("theta2_ORR_interim",1:L),paste0("theta2_TOX_interim",1:L),"sim_ind","case_id")
  stats_vec_means = apply(stats_vec[,c(1:7,10,10+L,9+2*L+2)],2,function(x) mean(as.numeric(x)))
  if(WithDetails==TRUE){
    return(stats_vec)
  }
  else{
    return(stats_vec_means)
  }
}


get_stats_vec_BOPII_ONLY = function(i,case_id,sim_array_High_Low_Compare_list,
                                    pi_with_phi_High_list,
                                    pi_with_phi_Low_list,stopBoundary_mat_opt,n){
  sim_array_High_Low_Compare_case = sim_array_High_Low_Compare_list[[case_id]]
  pi_with_phi_Low_case = pi_with_phi_Low_list[[case_id]]
  pi_with_phi_High_case = pi_with_phi_High_list[[case_id]]
  data_temp = sim_array_High_Low_Compare_case[,,i,]
  Termin_H = FALSE
  Termin_L = FALSE
  Termin_H_only = FALSE
  Termin_L_only = FALSE
  Termin_H_reason=NA
  Termin_L_reason=NA
  Termin_BOTH =FALSE
  Termin_None = FALSE
  n.sample.size_High_dose = 0
  n.sample.size_Low_dose = 0
  n.sample.size = 0
  for(L_i in 1:L){
    if(Termin_H==FALSE|Termin_L==FALSE){
      if(Termin_H ==FALSE){
        #Single BOPII
        High_temp = data_temp[,L_i,1]
        n.sample.size_High_dose =n.sample.size_High_dose+n
        if(High_temp%*%collapse_ORR<=stopBoundary_mat_opt[L_i,2] | 
           High_temp%*%collapse_TOX>=stopBoundary_mat_opt[L_i,3] ){
          Termin_H = TRUE
          Termin_H_reason= paste0("BOPII_termin_interim_",L_i)
        }
      }
      if(Termin_L ==FALSE){
        #Single BOPII
        Low_temp = data_temp[,L_i,2]
        n.sample.size_Low_dose =n.sample.size_Low_dose+n
        if(Low_temp%*%collapse_ORR<=stopBoundary_mat_opt[L_i,2] | 
           Low_temp%*%collapse_TOX>=stopBoundary_mat_opt[L_i,3] ){
          Termin_L = TRUE
          Termin_L_reason= paste0("BOPII_termin_interim_",L_i)
        }
      } ## Complete patients collection at L_i-th interim and single-arm BOPII
      
      if(L_i==L){
        Termin_None = (1-Termin_L)*(1-Termin_H)
        Termin_L_only =(Termin_L)*(1-Termin_H)
        Termin_H_only=(Termin_H)*(1-Termin_L)
        Termin_BOTH = (Termin_L)*(Termin_H)
        n.sample.size = n.sample.size + n.sample.size_High_dose+n.sample.size_Low_dose
      }
      
    }
    else{ ## When two doses are terminated already
      Termin_None = (1-Termin_L)*(1-Termin_H)
      Termin_L_only =(Termin_L)*(1-Termin_H)
      Termin_H_only=(Termin_H)*(1-Termin_L)
      Termin_BOTH = (Termin_L)*(Termin_H)
      n.sample.size = n.sample.size + n.sample.size_High_dose+n.sample.size_Low_dose
      break
    }
  }
  stats =c(Termin_L_only,Termin_H_only,Termin_None,Termin_BOTH,
           n.sample.size,n.sample.size_High_dose,n.sample.size_Low_dose,
           Termin_H_reason,Termin_L_reason,
           i,case_id)
  names(stats) = c('Termin_L_only','Termin_H_only','Termin_BOTH','Termin_None','n.sample.size',
                   "n_sample_size_High_dose","n_sample_size_Low_dose",
                   "Termin_H_reason","Termin_L_reason","num_sim","case")
  return(stats)
}




#### theta2 as a function of interim sample size
stopBoundary_cal_compare_theta2_interim_size=function(theta2_ORR,theta2_TOX,case_id,sim_array_High_Low_Compare_list,
                                            pi_with_phi_High_list,
                                            pi_with_phi_Low_list,stopBoundary_mat_opt,n){
  
  sim_array_High_Low_Compare_case = sim_array_High_Low_Compare_list[[case_id]]
  pi_with_phi_Low_case = pi_with_phi_Low_list[[case_id]]
  pi_with_phi_High_case = pi_with_phi_High_list[[case_id]]
  
  
  stats_vec=c()
  ## Need to soft code case_i
  for(i in 1:n.sim){
    data_temp = sim_array_High_Low_Compare_case[,,i,]
    
    Termin_H = FALSE
    Termin_L = FALSE
    Termin_H_only = FALSE
    Termin_L_only = FALSE
    Termin_BOTH =FALSE
    Termin_None = FALSE
    n.sample.size_High_dose = 0
    n.sample.size_Low_dose = 0
    n.sample.size = 0
    for(L_i in 1:L){
      if(Termin_H==FALSE|Termin_L==FALSE){
        if(Termin_H ==FALSE){
          #Single BOPII
          High_temp = data_temp[,L_i,1]
          n.sample.size_High_dose =n.sample.size_High_dose+n
          if(High_temp%*%collapse_ORR<=stopBoundary_mat_opt[L_i,2] | 
             High_temp%*%collapse_TOX>=stopBoundary_mat_opt[L_i,3] ){
            Termin_H = TRUE
          }
        }
        if(Termin_L ==FALSE){
          #Single BOPII
          Low_temp = data_temp[,L_i,2]
          n.sample.size_Low_dose =n.sample.size_Low_dose+n
          if(Low_temp%*%collapse_ORR<=stopBoundary_mat_opt[L_i,2] | 
             Low_temp%*%collapse_TOX>=stopBoundary_mat_opt[L_i,3] ){
            Termin_L = TRUE
          }
        } ## Complete patients collection at L_i-th interim and single-arm BOPII
        
        ## Comparison if both pass the futility test
        if(Termin_H==FALSE & Termin_L ==FALSE){
          ## Need to soft code pi_with_phi_1 and **_2.
          pos_prob_compare_ORR_temp = compare_pos_cal(eta = eta_ORR,x_1 = High_temp,x_2 =Low_temp,
                                                         collapse_type = collapse_ORR,
                                                         pi_with_phi_1 =  pi_with_phi_High_case,
                                                         pi_with_phi_2 =  pi_with_phi_Low_case,n0 = 1
          ) 
          # pos_prob_compare_ORR_temp is P(ORR_h - ORR_l > eta_ORR | X)
          
          pos_prob_compare_TOX_temp = 1-compare_pos_cal(eta = eta_TOX,x_1 = High_temp,x_2 =Low_temp,
                                                           collapse_type = collapse_TOX,
                                                           pi_with_phi_1 =  pi_with_phi_High_case,
                                                           pi_with_phi_2 =  pi_with_phi_Low_case,n0 = 1
          )
          # pos_prob_compare_TOX_temp is P(TOX_h - TOX_l < eta_TOX | X)
          
          if(pos_prob_compare_TOX_temp>theta2_TOX[L_i] & (pos_prob_compare_ORR_temp>theta2_ORR[L_i])){
            ## Low dose is terminated
            Termin_L = TRUE
          }
          if((1-pos_prob_compare_TOX_temp>theta2_TOX[L_i]) & (1-pos_prob_compare_ORR_temp>theta2_ORR[L_i])){
            ## High dose is terminated
            Termin_H = TRUE
          }
          
        }
        
        if(L_i==L){
          Termin_None = (1-Termin_L)*(1-Termin_H)
          Termin_L_only =(Termin_L)*(1-Termin_H)
          Termin_H_only=(Termin_H)*(1-Termin_L)
          Termin_BOTH = (Termin_L)*(Termin_H)
          n.sample.size = n.sample.size + n.sample.size_High_dose+n.sample.size_Low_dose
        }
        
      }
      else{ ## When two doses are terminated already
        Termin_None = (1-Termin_L)*(1-Termin_H)
        Termin_L_only =(Termin_L)*(1-Termin_H)
        Termin_H_only=(Termin_H)*(1-Termin_L)
        Termin_BOTH = (Termin_L)*(Termin_H)
        n.sample.size = n.sample.size + n.sample.size_High_dose+n.sample.size_Low_dose
        break
      }
    }
    stats =c(Termin_L_only,Termin_H_only,Termin_None,Termin_BOTH,n.sample.size,n.sample.size_High_dose,n.sample.size_Low_dose)
    stats_vec = rbind(stats_vec,stats)
  }
  
  
  colnames(stats_vec) = c("Recommend High only","Recommend Low only","Recommend Both","Recommend None",
                          "n_sample_size","n_sample_size_High_dose","n_sample_size_Low_dose")
  stats_vec_means =apply(stats_vec,2,mean)
  return(stats_vec_means)
}
